/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DynlgMemSP.c
 *
 *   Path    $(TESTSUITEROOT)/Behaviour/DynLink/
 *
 *   Desc:
 *           Checks the Bridge behaviour when a node with split
 *			 phases, each phase having large memory
 *			 sections for data and text is loaded.
 *
 *  Usage:
 *        DYNLGMEMSP [-v] [-l "log path/filename"]
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -n: Number of Transfers for each Iteration
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 18, 2003    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include "dynLink.h"

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define DEFNUMBUFS         1
#define DEFNUMTXFRS    10000
#define NUMOFCOMPARGS      2	/* Compulsory args: n,s */
#define PASS               0
#define FAIL               1

#define SHORT short

#define DSPEXEC         "dataxfr_nortdxidl.dof"
#define DLLNAME         "/release/inoutlgmemsp.dll55l"
#define CREATEPHDLLNAME  "/release/inoutlgmemspCreate.dll55l"
#define EXECPHDLLNAME    "/release/inoutlgmemspExecute.dll55l"
#define DELETEPHDLLNAME  "/release/inoutlgmemspDelete.dll55l"




/* Function to exit gracefully */
static INT graceful_exit()
{
	TST_verbose("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);
}

/* Function for Large mem footprint check test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents. Copy the contents to local variables    *
 * and use	*
 */
static INT lgmemSPCheckFxn(dynLinkArgs * targs)
{

    struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode = NULL;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    struct DSP_CBDATA				*cbData;
	DSP_STATUS				status;
	DSP_STATUS				exitStatus = DSP_SOK;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
	 BYTE                            argsBuf[32 + sizeof(ULONG)];
	 INT				threadID ;

	 BYTE				**arrBuf1;
	 CHAR				* dllName;

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	threadID = targs->ithrdNum ;
	pszUuid = targs->uuidPtr;
	dllName = targs->dllName;


    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = targs->transferMode;
    attrs.uNumBufs = DEFNUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

	if (targs->transferMode  == STRMMODE_PROCCOPY)
    {
		attrs.uSegid = 0;
    }
    else
    {
		attrs.uSegid = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }

    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


   cbData = (struct DSP_CBDATA *)argsBuf ;

   cbData->cbData = 32;
	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);
	arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);


	// pszUuid is passed from calling function
	//printf("UUID is = %s\n",pszUuid);
	//printf("DLL  is = %s\n",dllName);
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

	//printf("STATUS DSPNode_Allocate = 0x%x\n",status);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        return FAIL;
    }

    //printf("After Allocate\n");

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);
	//printf("STATUS DSPNode_Connect 1 = 0x%x\n",status);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    //printf("STATUS DSPNode_Connect 2 = 0x%x\n",status);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    //printf("After Connect\n");

    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (status != DSP_EDYNLOAD)
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    if (status == DSP_EDYNLOAD)
    {
        return PASS;
    }
    else
    {
        return FAIL;
    }

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);

    //printf("STATUS DSPNode_Run = 0x%x\n",status);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }

    //printf("After Run\n");


    status = DSPNode_Terminate(hNode, &exitStatus);
	if (!DSP_SUCCEEDED(status))
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog)
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Terminate()\n");
		}
		return FAIL;
	}

    status = DSPNode_Delete(hNode);
	if (!DSP_SUCCEEDED(status))
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog)
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Delete()\n");
		}
		return FAIL;
	}

    /* Return from this Thread */
    if (DSP_SUCCEEDED(status))
    {
        return PASS;
    }
    else
    {
        return FAIL;
    }
}



/*
 *  ======== main ========
 *
 */
INT dynLgMemSP_BEHAVIOUR(INT argc, CHAR * argv[])
{
    BOOL           			bUseLog=FALSE;

    BOOL           			bVerbose=FALSE;
    FILE           			*pHandle = NULL;

    INT            			iNumTransfers = DEFNUMTXFRS;
    INT            			iBuffSize = DEFBUFSIZE;

	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	dynLinkArgs 			funcArgs;
    struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
    INT                 	retVal = PASS;
	INT 					transferMode = STRMMODE_PROCCOPY;
	CHAR					*uuidPtr = "27C58E71_C057_4E07_8126_5FF867CE903B"; // inoutlgmemsp
	CHAR        			*aDllName[DEFNAMESIZE] = { DLLNAME, "arg3" };
	struct DSP_UUID				gidNodeID;
	CHAR					*creatPhDll = CREATEPHDLLNAME;
	CHAR					*execPhDll = EXECPHDLLNAME;
	CHAR					*delPhDll = DELETEPHDLLNAME;
	BOOL					fLoadDsp = FALSE;



	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Call the lgMemCheck func and pass the
	   params to this function */
	/* In the Function do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set Properties, dynamically Allocate,Create the Node on DSP */
		/* Calling create calls the node with large mem footprint */
		/* Open the Streams */
		/* Run the task */
		/* Do the Data Transfer */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DynamicLink_BEHAVIOUR: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "dynLink test:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
			break;
		case 'd':
        	/* use given dll */
        	aDllName[0] = (char *) optarg;
            break;
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to Processor */
    status = DSPProcessor_Attach(0, pAttrIn, &hProcessor);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status))
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
    }
    }

    UUID_UuidFromString((CHAR *)uuidPtr, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DSP_DCDNODETYPE, aDllName[0]);
    if (!DSP_SUCCEEDED(status))
	{
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
        }
        return FAIL;
    }

    status = DSPManager_RegisterObject(&gidNodeID,DSP_DCDCREATELIBTYPE,creatPhDll);
    status = DSPManager_RegisterObject(&gidNodeID,DSP_DCDEXECUTELIBTYPE,execPhDll);
    status = DSPManager_RegisterObject(&gidNodeID,DSP_DCDDELETELIBTYPE,delPhDll);

    // status = DSPManager_RegisterObject(&gidNodeID, DSP_DCDLIBRARYTYPE, aDllName[0]);
    if (!DSP_SUCCEEDED(status))
	{
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
        }
        return FAIL;
    }

	/* Fill the funcArgs which are to be passed on to the Thread */
	funcArgs.hProcessor = hProcessor;
	funcArgs.iNumTransfers = iNumTransfers;
	funcArgs.iBuffSize = iBuffSize;
	funcArgs.pHandle = pHandle;
	funcArgs.bUseLog = bUseLog;
	funcArgs.bVerbose = bVerbose;
	funcArgs.dllName = aDllName[0];
	funcArgs.uuidPtr = uuidPtr;

	funcArgs.transferMode = transferMode;
	retVal = lgmemSPCheckFxn(&funcArgs);

    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS)
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

