/*
*  Copyright 2004 by Texas Instruments Incorporated.
*  All rights reserved. Property of Texas Instruments Incorporated.
*  Restricted rights to use, duplicate or disclose this code are
*  granted through contract.
*
*  @(#) DSP/BIOS Bridge 
*/
/*  ============================================================================
*   File    NodeAttr.c
*
*   Path    $(TESTSUITEROOT)\Behaviour\Node\
* 
*   Desc:
*        Test to ennumerate nodes, query the attrs and get the execution state
*
*  Usage:
*        NODEATTR [-v] [-l "log path/filename"]
*                       [-x "valid executable"]
*                       [-m "message to be sent"]
*
*        -v: verbose mode
*        -l: use log and location
*        -x: name of valid DSP executable
*        -n: Number of nodes to create
*
*   ---------------------------------------------------------------------------
*   Rev     1.0.0
* 
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   DEC 05, 2002    REF=ORG             Swamy MC
*   ============================================================================
*/


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include <Flush.h>

#define DEFNAMESIZE    80
#define MAXNUMOFNODES   4

#define DSPEXEC         "dataxfr_nortdxidl.dof"

/*
*  ======== nodeAttr_BEHAVIOUR ========
*  Create a number of nodes with different attrs
*  Ennumerate the nodes and then for each of the node,
*  - Get the attrs and execution state
*
*/
INT nodeAttr_BEHAVIOUR(INT argc, CHAR * argv[], INT iRunTime)
{
    
    INT                 opt;
    INT                 cRetVal = 1;
    INT                 iArgc = 1;
    INT                 i;
    BOOL                bUseLog=FALSE;
    FILE                *pHandle = NULL;
    CHAR                szLogName[DEFNAMESIZE];
    CHAR                *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR                *aEnvp[DEFNAMESIZE] = {NULL};
    CHAR                *pszUuid;
    DSP_HPROCESSOR      hProcessor;
    struct DSP_UUID            gidNodeID;
    DSP_STATUS          status;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    struct DSP_NODEATTRIN      *pNodeAttrin;
    struct DSP_CBDATA          *cbData;
    DSP_HNODE           hNodes[MAXNUMOFNODES];
    struct DSP_NODEATTR        nodeAttr;
    struct DSP_STRMATTR        attrs;
    DSP_HNODE           aNodeTab[MAXNUMOFNODES];
    UINT                uNodeTabSize;
    INT                 iNodesToCreate = MAXNUMOFNODES;
    UINT                uNumNodes;
    UINT                uAllocated;
    BOOL                fLoadDsp = FALSE;
    
    
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;
    
    pNodeAttrin = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    pNodeAttrin->cbStruct = sizeof(struct DSP_NODEATTRIN);
    pNodeAttrin->uTimeout = 1000;
    pNodeAttrin->iPriority = 1;
    

#if defined(OMAP_2430)|| defined(OMAP_3430)
   pNodeAttrin->uProfileID = 0xff;
#endif
    
    cbData = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    cbData->cbData = 0;
    
    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = 512;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;
    
    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:m:n:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'n':
            /* Number of nodes to be created */
            iNodesToCreate = CSL_Atoi(optarg);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                    "Invalid Option Specified.\n");
            }
            break;
        }
    }
    
    if (cRetVal > 0) 
    {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
        
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }
        
        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
            
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
        
        if (DSP_FAILED(status)) 
        {
            /* Report Error */
            cRetVal = 0;
            if (bUseLog) 
            {
                TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Attach. Status  = 0x%x\n", status);
            }
            return 1;
        }

        if(fLoadDsp)
        {
            /* Load Processor with Executable */
            status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, 
                (CONST CHAR **)aEnvp);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                cRetVal = 0;
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. Status  = 0x%x\n", status);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
            
            /* Start Processor */
            status = DSPProcessor_Start(hProcessor);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                cRetVal = 0;
                
                TST_verbose("FAIL\n");
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. Status  = 0x%x\n", status);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
        }
        
        pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1";
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);
        
        for (i=0; i<iNodesToCreate; i++)
        {
            pNodeAttrin->uTimeout = ((i+1) * 1000);
            pNodeAttrin->iPriority = i+1;
            
            status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, pNodeAttrin, &hNodes[i]);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                cRetVal = 0;
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate. Status  = 0x%x\n", status);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
            
            status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNodes[i], 0,
                &attrs);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect. Status  = 0x%x\n", status);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
            
            status = DSPNode_Connect(hNodes[i], 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                &attrs);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect. Status  = 0x%x\n", status);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
        }   /* End of for loop */
        /* Ennumerate the nodes now */
        uNodeTabSize = sizeof(DSP_HNODE) * MAXNUMOFNODES;
        status = DSPProcessor_EnumNodes(hProcessor, &aNodeTab[0], uNodeTabSize, 
            &uNumNodes, &uAllocated);
            /* Check if the num of nodes returned is the same as the num of 
        nodes allocated */
        if (uNumNodes != iNodesToCreate)
        {
            /* Report Error */
            if (bUseLog) 
            {
                TSTLOG_appendLog(pHandle,"Mismatch: Number of Nodes Allocated = %d and ennumerated = %d\n",
                    uNumNodes,iNodesToCreate);
            }
            DSPProcessor_Detach(hProcessor);
            return 1;
        }
        for (i=0; i<uNumNodes; i++)
        {
            status = DSPNode_GetAttr(aNodeTab[i], &nodeAttr, sizeof(struct DSP_NODEATTR));
            /* Check if the Node is in Allocated state */
            if (nodeAttr.iNodeInfo.nsExecutionState != NODE_ALLOCATED)
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,
                        "Node state is NOT Allocated !! for Node Handle 0x%x\n",aNodeTab[i]);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
            /* Also check if the Priority of the node is okay */
            if (aNodeTab[i] != hNodes[(nodeAttr.inNodeAttrIn.iPriority -1)])
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,
                        "Priority for this node is not the same as allocated\n");
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
            
        } /* end of for loop */
        
        
        for (i=0; i<iNodesToCreate; i++)
        {
            /* Create Node on DSP */
            status = DSPNode_Create(hNodes[i]);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPNode_Create. Status  = 0x%x\n", status);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
            
            status = DSPNode_GetAttr(aNodeTab[i], &nodeAttr, sizeof(struct DSP_NODEATTR));
            /* Check if the Node is in Created state */
            if (nodeAttr.iNodeInfo.nsExecutionState != NODE_CREATED)
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,
                        "Node state is NOT Allocated !! for Node Handle 0x%x\n",aNodeTab[i]);
                }
                DSPProcessor_Detach(hProcessor);
                return 1;
            }
            
        }
    } /* End of if cRetVal */
    
    status = DSPProcessor_Detach(hProcessor);
    
    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(pNodeAttrin);
    OsSpecificMemFree(cbData);
    
    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }
    
    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    
    return 0;
}

