/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    framework.c
 *
 *   Path    $(TESTSUITEROOT)\Framework\framework.c
 * 
 *   Desc:
 *           Framework and Environment for Automated Test Suite Execution
 *           of all modules of DSP/BIOS Bridge for Linux and Nucleus (in future) 
 *           "framework -v" will run all tests verbose.
 *           "framework -l output.txt" will run all tests and store output to 
 *                   a file called "output.txt"
 *
 *   Usage:
 *           framework [-v] [-l "log path/filename"] 
 *               [-t Type of Test Suite to be executed]
 *               [-x DSP executable ]
 *               [-m module to test (or ALL for all) ]
 *               [-n list of API functions to test for 
 *                   particular module or "ALL" for all functions ]
 *               [-# number of times ]
 *
 *           -v: verbose mode
 *           -t: Type of Test Suite to be executed
 *                  API - For testing the APIs
 *                  ANALYSIS - Performance tests
 *                  STRESS - stressing the system
 *                  BEHAVIOUR - state transitions
 *           -l: use log and location to store results
 *           -x: DSP executable to be used in testing
 *           -m: individual module to test or ALL for all modules
 *               (Default = ALL)
 *           -n: list of API functions (see below) or ALL for all functions
 *               (Default = ALL)
 *           -#: number of times to run each API function test
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <tstlog.h>
#include <testosal.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include "framework.h"       /* Header file for Framework */

#define DEFNAMESIZE     80
#define SEPARATORS      " ,\t\n"
#define NUMDSPPROCESSOR 9
#define NUMDSPMANAGER   2
#define NUMDSPNODE      15
#define NUMDSPSTREAM    12
#define NUMNODE         5
#define NUMSTRM         8       
#define FAILURE         1     // 0=success
#define MAXCMDSIZE      200
#define MAXCMDS         200
#define MAXARGSIZE      50

#define ROOT_ACCESS	 1406

#define CSL_Strcmp strcmp

#define	TESTDRIVER_NAME	    "/dev/dummydriver"

/* Declaration of Static functions */
INT run_test_kern(INT acount, CHAR ** avars, INT TestDriverHandle);

/* Static function declarations */
static void DummyTrap(INT TestDriverHandle, INT tmp_argc, CHAR **tmp_argv);

/*
 * Function   : run_test
 * Description: Runs the test case. Based on the first parameter
 *              passed, the appropriated test suite is invoked and
 *              the remaining parameters are passed to that test suite
 * Parameters : (IN)argc - number of args
 *              (IN)argv - array of arguments
 * Return Val : None ??? Can add fail/success and log it in the calling func
 *
 */

INT run_test(INT acount, CHAR ** avars)
{

    CHAR        * tmpTstType = "ALL";
    CHAR        ** tmp_argv;
    INT         tmp_argc;
	INT count;


	tmpTstType = avars[0];

	/* Now drop the first parameter and pass the rest of the
	parameters to corresponding Test Suite */
	tmp_argc = acount;
	tmp_argv = avars;    
	printf("LOG: TESTARGS: ");


	for (count=0; count < acount; count++)
	{
		printf("%s  ",tmp_argv[count]);
	}
	printf("\n");


	if (!CSL_Strcmp(tmpTstType, "ALL"))
	{
		printf("\nIMPROPER Test Type specified !!!!!!!!!!!!! \n");
	}
	else if (!CSL_Strcmp(tmpTstType, "API"))
	{
		apitst_main(acount, avars);
	}
	else if (!CSL_Strcmp(tmpTstType, "ANALYSIS"))
	{
		anlysis_main(tmp_argc, tmp_argv);
	}
	else if (!CSL_Strcmp(tmpTstType, "BEHAVIOUR"))
	{
		behvr_main(tmp_argc, tmp_argv);
	}
	else if (!CSL_Strcmp(tmpTstType, "STRESS"))
	{
		stress_main(tmp_argc, tmp_argv);
	}
	else
	{
		printf("\n INVALID Test Type specified: %s\n", tmpTstType);
	}

	return 0;
}
/* end of run_test */

/*
 * Function   : extract_tokens
 * Description: Extracts the tokens from the command line and fills
 *              command arguments array, which is passed to run_test
 *              The function also counts the number of arguments
 * Parameters : (IN)command_line - command line
 *              (OUT)tmp_argc - number of arguments
 *              (out)cmd_args - arguments array
 * Return Val : None ??? Can add fail/success and log it in the calling func
 *
 */

// INT extract_tokens(char *command_line,INT *tmp_argc,CHAR **cmd_args)
INT extract_tokens(char *command_line,INT *tmp_argc,CHAR *cmd_args[])
{
    CHAR *token;
    INT i=0;

    // initialise num of args
    *tmp_argc = 0;
    token = strtok(command_line," \t");
    while(token != NULL)
    {
		cmd_args[i] = token;
        printf("Command arg %d : %s\n",i,cmd_args[i]);
        (*tmp_argc)++;
		i++;
		token = strtok(NULL," \t");
    }
    return 0;
}
/* end of extract_tokens */

static void DummyTrap(INT TestDriverHandle, INT tmp_argc, CHAR **tmp_argv)
{
	ioctl(TestDriverHandle, tmp_argc, tmp_argv);
}

static void graceful_exit(int ret_val)
{
	printf("Hit enter to exit:  ");
	(void)getchar();            

	exit (ret_val);
}


/* 
 *  ======== main function for Test Framework ======== 
 */
INT main (INT argc, CHAR * argv[])
{
    CHAR        * tmpTstType = "ALL";
    CHAR        ** tmp_argv;
    CHAR        *cmd_args[MAXCMDS];
    INT         tmp_argc;
    FILE        *cmds_file;
    CHAR        *command_line;
    INT         status;
	INT			TestDriverHandle;	
   
    status = DspManager_Open(ROOT_ACCESS, NULL);
    if (DSP_FAILED(status))    /* see dbapi.c for value */
    {
    	printf("\nFATAL ERROR: Bridge Initialisation FAILED, status = %x\n", status);
	    return status;
    }

    TestDriverHandle = open(TESTDRIVER_NAME,    2);
	
    CSL_Init();   


/* Handling multiple test suites/cases at the same time :
   IF the first parameter passed is "-f" then, the second param is the
   file name which contains the commands. Read this file and scan each
   line for commands. These are executed till EOF
*/

// If first param is "-f", check if num of params is 3. If more error.
// open the file specified and if open fails, error.
// once the file is open, go to a while loop which does:
// --> read line (till EOF)
// --> pass it on to the func which takes care of command

    tmpTstType = argv[1];

    if (!CSL_Strcmp(tmpTstType, "-f"))
    {
        printf("\nINFO: In the IF part: trying to run batch mode\n");
        // check if num of params is 3
        if (argc != 3)
        {
            // error - Incorrect number of parameters
            printf("\nERROR: incorrect num of params: %d \n",argc);
            graceful_exit(0);
        }
        cmds_file = fopen(argv[2],"r");
        if (cmds_file == NULL)
        {

            // error - could not open the file for reading
            printf("\nERROR: could not open file: %s for reading\n",argv[2]);
            graceful_exit(0);
        }

		command_line = OsSpecificMemAlloc(MAXCMDSIZE);
		if (command_line == NULL)
		{
			printf("\nERROR: Memory Allocation for command_line failed !!\n");
            graceful_exit(0);
        }

        while(fgets(command_line,MAXCMDSIZE,cmds_file) != NULL)
        {

            // find the number of args and pass argcount and args
            // to run_test function
			printf("\nINFO: Command Line: %s \n",command_line);
            status = extract_tokens(command_line, &tmp_argc, cmd_args);
            if (status != 0)
            {
                printf("\nERROR: could not Count number of args\n");
                graceful_exit(0);
            }

            // Call the Test case/function
            status = run_test(tmp_argc,cmd_args);

			OsSpecificMemFree(command_line);  // free the mem after running the test
			command_line = OsSpecificMemAlloc(MAXCMDSIZE); // for the next test case !!!
			if (command_line == NULL)
			{
				printf("\nERROR: Memory Allocation for command_line failed !!\n");
			    graceful_exit(0);
			}

        }
        fclose(cmds_file);
    }
    else if (!CSL_Strcmp(tmpTstType, "-k"))
    {
		/* Now drop the first parameter and pass the rest of the
		   parameters to corresponding Test Suite */
		tmp_argc = argc;  tmp_argc--; tmp_argc--;
		argv++; argv++;  tmp_argv = argv;
        status = run_test_kern(tmp_argc,tmp_argv,TestDriverHandle);
    }
    else
    {
		/* Now drop the first parameter and pass the rest of the
		   parameters to corresponding Test Suite */
		tmp_argc = argc;  tmp_argc--;
		argv++;  tmp_argv = argv;
        status = run_test(tmp_argc,tmp_argv);
    }


    CSL_Exit();

    status = DspManager_Close(0, NULL);
    if (DSP_FAILED(status))    /* see dbapi.c for value */
    {
    	printf("\nERROR: Bridge Close FAILED, status = %x\n", status);
    }

    return status;
}

/* Runs the test case in Kernel mode */
INT run_test_kern(INT acount, CHAR ** avars, INT TestDriverHandle)
{

    CHAR        * tmpTstType = "ALL";
    CHAR        ** tmp_argv;
    INT         tmp_argc;

	tmpTstType = avars[0];

	/* Now drop the first parameter and pass the rest of the
	parameters to corresponding Test Suite */
	tmp_argc = acount;
	tmp_argv = avars;

	DummyTrap(TestDriverHandle, tmp_argc, tmp_argv);
	return 0;
}
/* end of run_test */

