/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*
 *  ======== cslce.c ========
 *  Purpose:
 *      Provides platform independent C Standard library functions.
 *
 *  Public Functions:
 *      CSL_AnsiToWchar
 *      CSL_Atoi
 *      CSL_ByteSwap
 *      CSL_Exit
 *      CSL_Init
 *      CSL_NumToAscii
 *      CSL_Strcmp
 *      CSL_Strcpyn
 *      CSL_Strlen
 *      CSL_Strncat
 *      CSL_Strncmp
 *      CSL_Strtok
 *      CSL_WcharToAnsi
 *      CSL_Wstrlen
 *
 *  Notes:
 *      As of 29-Dec-1997, this module is platform independant.  This means,
 *      no platform dependant OS service calls can be made from this module.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *! Revision History:
 *! ================
 *! 21-Sep-2001 jeh: Added CSL_Strncmp(). Alphabetized functions.
 *! 22-Nov-2000 map: Added CSL_Atoi and CSL_Strtok
 *! 19-Nov-2000 kc: Added CSL_ByteSwap.
 *! 09-Nov-2000 kc: Added CSL_Strncat. 
 *! 03-Feb-2000 rr: Module init/exit is handled by OSAL Init/Exit.GT Changes.
 *! 15-Dec-1999 ag: Removed incorrect assertion CSL_NumToAscii()
 *! 29-Oct-1999 kc: Added CSL_Wstrlen for UNICODE strings.
 *! 30-Sep-1999 ag: Removed DBC assertion (!CSL_DebugMask.flags) in 
 *                  CSP_Init().
 *! 20-Sep-1999 ag: Added CSL_WcharToAnsi().
 *!                 Removed call to GT_set().
 *! 19-Jan-1998 cr: Code review cleanup.
 *! 29-Dec-1997 cr: Made platform independant, using MS CRT code, and 
 *!                 combined csl32.c csl95.c and cslnt.c into csl.c.  Also 
 *!                 changed CSL_lowercase to CSL_Uppercase.
 *! 21-Aug-1997 gp: Fix to CSL_strcpyn to initialize Source string, the NT way.
 *! 25-Jun-1997 cr: Created from csl95, added CSL_strcmp.
 */

/* ----------------------------------- Linux OS */
#include <os.h>
#include <stdio.h>

/*  ----------------------------------- DSP/BIOS Bridge */
#include <std.h>
#include <dbdefs.h>

/*  ----------------------------------- This */
#include <testcsl.h>


/*  ----------------------------------- Globals */
static ULONG cRefs = 0;                 /* module reference counter */


/* ----------------------------------- Local Declarations */
DWORD CSL_Strlen(IN CONST PSTR );

/*
 *  ======== CSL_AnsiToWchar ========
 *  Purpose:
 *  Convert an ansi string to a wide CHAR string.
 */
ULONG CSL_AnsiToWchar(OUT WCHAR *pwszDest, IN PSTR pstrSource, IN ULONG uSize)
{
    DWORD   dwLength;
    DWORD   i;
    
    if (!pstrSource) {
        return (0);
    }
    
    dwLength = CSL_Strlen(pstrSource);
    for (i = 0; (i < dwLength) && (i < (uSize - 1)); i++) {
        pwszDest[i] = pstrSource[i];
    }
    pwszDest[i] = '\0';
    
    return (i);
}


/*
 *  ======= CSL_Atoi =======
 *  Purpose:
 *      Convert a string to an integer
 */
INT CSL_Atoi(IN CONST CHAR *ptstrSrc)
{
    char *end_position ;

#ifndef __KERNEL__
    return strtol(ptstrSrc, &end_position, 10);
#else
    return simple_strtol(ptstrSrc, &end_position, 10);
#endif
}


/*
 *  ======== CSL_ByteSwap ========
 *  Purpose:
 *      Swap bytes in buffer. 
 */
VOID CSL_ByteSwap(IN PSTR pstrSrc, OUT PSTR pstrDest, IN ULONG ulBytes)
{
    int i ;
    
    /* swap bytes. */
    for (i=0; i<ulBytes/2 ; i++)
    {
        pstrDest[2*i] = pstrSrc[2*i + 1] ;
        pstrDest[2*i + 1] = pstrSrc[2*i] ;
    }
}


/*
 *  ======== CSL_Exit ========
 *  Purpose:
 *      Discontinue usage of the CSL module.
 */
void CSL_Exit()
{
    
    cRefs--;
    
}


/*
 *  ======== CSL_Init ========
 *  Purpose:
 *      Initialize the CSL module's private state.
 */
BOOL CSL_Init()
{
    
    cRefs++;

    return (TRUE);
}


/*
 *  ======== CSL_NumToAscii ========
 *  Purpose:
 *      Convert a 1 or 2 digit number to a 2 digit string.
 */
VOID CSL_NumToAscii(OUT PSTR pstrNumber, DWORD dwNum)
{
    CHAR    tens;
    
    tens = (CHAR)dwNum / 10;
    dwNum = dwNum % 10;

    if (tens) {
        pstrNumber[0] = tens + '0';
        pstrNumber[1] = (CHAR)dwNum + '0';
        pstrNumber[2] = '\0';
    }
    else {
        pstrNumber[0] = (CHAR)dwNum + '0';
        pstrNumber[1] = '\0';
    }
}


/*
 *  ======== CSL_Strcmp ========
 *  Purpose:
 *      Compare 2 ASCII strings.  Works the same was as stdio's strcmp.
 *  Details:
 *      This code was taken from MS CRT src code for strcmp.
 */
LONG CSL_Strcmp(IN CONST PSTR pstrStr1, IN CONST PSTR pstrStr2)
{
    INT             ret = 0;
    CONST CHAR      * src = pstrStr1;
    CONST CHAR      * dst = pstrStr2;
    
    while (!(ret = *(UCHAR *)src - *(UCHAR *)dst) && *dst) {
        ++src, ++dst;
    }
    
    if (ret < 0) {
        ret = -1;
    }
    else if (ret > 0) {
        ret = 1;
    }
    
    return (ret);
}


/*
 *  ======== CSL_Strcpyn ========
 *  Purpose:
 *      Safe strcpy function.
 */
PSTR CSL_Strcpyn(OUT PSTR pstrDest, IN CONST PSTR pstrSrc, DWORD cMax)
{
    CHAR            * dest = pstrDest;
    CONST CHAR      * source = pstrSrc;
    CHAR            * start = dest;

    while (cMax && (*dest++ = *source++)) {   /* copy string */
        cMax--;
    }

    if (cMax) {
        /* pad out with zeroes */
        while (--cMax) {
            *dest++ = '\0';
        }
    }
    return (start);
}


/*
 *  ======== CSL_Strlen ========
 *  Purpose:
 *      Determine the length of a null terminated ASCII string.
 */
DWORD CSL_Strlen(IN CONST PSTR pstrSrc)
{
    /* from MS CRT source to strlen */
    CONST CHAR  * pStr = pstrSrc;
    
    while (*pStr++);

    return((DWORD)(pStr - pstrSrc - 1));
}


/*
 *  ======== CSL_Strncat ========
 *  Purpose:
 *      Concatenate two strings together
 */
PSTR CSL_Strncat(IN PSTR pszDest, IN PSTR pszSrc, IN DWORD dwSize)
{
    return (strncat(pszDest, pszSrc, dwSize));
}


/*
 *  ======== CSL_Strncmp ========
 *  Purpose:
 *      Compare at most n characters of two ASCII strings.  Works the same
 *      way as stdio's strncmp.
 */
LONG CSL_Strncmp(IN CONST PSTR pstrStr1, IN CONST PSTR pstrStr2, DWORD n)
{
    INT             ret = 0;
    CONST CHAR      * src = pstrStr1;
    CONST CHAR      * dst = pstrStr2;
    ULONG           i = 0;
    
    while (i < n && !(ret = *(UCHAR *)src - *(UCHAR *)dst)
        && *src && *dst) {
        ++src, ++dst;
        ++i;
    }
    
    if (ret < 0) {
        ret = -1;
    }
    else if (ret > 0) {
        ret = 1;
    }

    return (ret);
}


/*
 *  ======= CSL_Strtok =======
 *  Purpose:
 *      Tokenize an input string based on the separators
 *      Not implemented in Linux because strsep requires first argument to be 
 *      CHAR **. This function is not called from Bridge.
 */
CHAR * CSL_Strtok(IN CHAR *ptstrSrc, IN CONST CHAR *szSeparators)
{
    return NULL;
}


/*
 *  ======== CSL_WcharToAnsi ========
 *  Purpose:
 *      UniCode to Ansi conversion.
 *  Note: 
 *      uSize is # of chars in destination buffer.
 */
ULONG CSL_WcharToAnsi(OUT PSTR pstrDest,
                                   IN WCHAR *pwszSource, 
                                   ULONG uSize)
{
    PSTR    pstrTemp = pstrDest;
    ULONG   uNumOfChars = 0;   
    
    if (!pwszSource) {
        return (0);
    }
    
    while ((*pwszSource != (WCHAR)'\0') && (uSize-- > 0)) {
        *pstrTemp++= (CHAR)*pwszSource++;
        uNumOfChars++;
    }
    *pstrTemp = '\0';
    
    return (uNumOfChars);
}


/*
 *  ======== CSL_Wstrlen ========
 *  Purpose:
 *      Determine the length of a null terminated UNICODE string.
 */
DWORD CSL_Wstrlen(IN CONST TCHAR *ptstrSrc)
{
    CONST TCHAR     * ptstr = ptstrSrc;
    
    while (*ptstr++);

    return((DWORD)(ptstr - ptstrSrc - 1));
}

