/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    MultiThreadData.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\MultiThread\MultiThreadData.c
 * 
 *   Desc:
 *           This test case tests the Data Transfer by 
 *           introducing delays at certain points during Data Transfer.
 *
 *  Usage:
 *        DelayTransfer [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -t: Number of Threads to be created
 *        -n: Number of Transfers for each Iteration
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 05, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <semaphore.h>
#include <DSPProcessor_OEM.h>
#include "MultiThread.h"

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define DEFNUMBUFS         1
#define DEFNUMTXFRS    10000
#define DEFNUMTHREADS     10 
#define NUMOFCOMPARGS      3	/* Compulsory args: t,n,s */
#define MAXNUMOFTHREADS    10    /* This number taken from Symbian Test Suite */
#define PASS               0
#define FAIL               1
#define DEFAULTDSPUNIT     0
#define SHORT short

#define DSPEXEC        "mttask_nortdxidl.dof"
static sem_t semCreate = { {0,0}, 1 };


/* Function to exit gracefully */
static INT graceful_exit()
{
	/*printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);*/
}

/* Thread function for Multi thread data transfer test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents as this will be used/needed by all the	*
 * threads created. Copy the contents to local variables and use	*
 */
static INT threadFxn(MultiThreadArgs * targs)
{

	struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf;
    	struct DSP_NODEATTRIN			nodeAttrin;
    	DSP_HNODE				hNode = NULL;
	DSP_HSTREAM				hStream;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    	struct DSP_CBDATA				*cbData;
	DSP_STATUS				status;
	INT						i,k;
	DSP_STATUS				exitStatus = DSP_SOK;
	struct DSP_STREAMATTRIN        strmAttrs;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
	BYTE                            argsBuf[32 + sizeof(ULONG)];
	DWORD				bufSize ;
	DWORD				dataSize ;
	DWORD				dwArg ;
	INT				threadID ;
	SHORT				* pData ;

	DSP_HSTREAM		hStream1;
	BYTE				**arrBuf1;
	SHORT *			sendBuffer ;
	SHORT *			receiveBuffer ;

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	threadID = targs->ithrdNum ;

	sem_post(&semCreate);


    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = targs->transferMode;
    attrs.uNumBufs = DEFNUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

	if (targs->transferMode  == STRMMODE_PROCCOPY)
    {
		attrs.uSegid = 0;
    }
    else
    {
		attrs.uSegid = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }
    
    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

   cbData = (struct DSP_CBDATA *)argsBuf ;    
 
   cbData->cbData = 32;
	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);
	arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);


	pszUuid = "F95CA07A_EE7D_11D5_A8DD_00B0D055F6D1";  // taken from mttask_nortdx_int.cdb file
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }


	strmAttrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
	strmAttrs.uTimeout = 5000;
	strmAttrs.uAlignment = 0;
	strmAttrs.uNumBufs = DEFNUMBUFS;   /* To have 4 buffers in the Queue, for this test case */
	strmAttrs.lMode = targs->transferMode ;
	strmAttrs.uDMAChnlId = 0;
	strmAttrs.uDMAPriority = 0;

	if (strmAttrs.lMode  == STRMMODE_PROCCOPY)
        {
        	strmAttrs.uSegment = 0;
        }
        else
        {
	    strmAttrs.uSegment = DSP_SHMSEG0;    /* Use shared memory segment 0 */
        }


    status = DSPStream_Open(hNode, DSP_TONODE, 0, &strmAttrs, &hStream);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hStream, iBuffSize, arrBuf, DEFNUMBUFS);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        return FAIL;
    }

    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, &strmAttrs, &hStream1);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hStream1, iBuffSize, arrBuf1, DEFNUMBUFS);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        return FAIL;
    }

	
        pData = (SHORT *)arrBuf[0] ;
        receiveBuffer = (SHORT *)arrBuf1[0] ;

        /*  Data transfer for number of iterations .. specified by the
         *  user as parameters to the test ... set in the constructor
	     */
        sendBuffer = OsSpecificMemAlloc(iBuffSize) ;

        for (k = 0 ; k < iNumTransfers ; k++)
	    {
		    for(i=0;i<iBuffSize/2;i++)
            {
	            pData[i]  = sendBuffer[i] = threadID + i + k ;
	        }

 			/* Issue Full Data Buffer */
			status = DSPStream_Issue(hStream, (BYTE *)pData, iBuffSize, iBuffSize, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				return FAIL;
			}

			/* Reclaim Empty Buffer */
			status = DSPStream_Reclaim(hStream, (BYTE **)&pData, &dataSize, &bufSize,&dwArg);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				return FAIL;
			}

			/* Issue Empty Buffer */
			status = DSPStream_Issue(hStream1, (BYTE *)receiveBuffer, iBuffSize, iBuffSize, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				return FAIL;
			}

			/* Reclaim Full Buffer */
			status = DSPStream_Reclaim(hStream1, (BYTE **)&receiveBuffer, &dataSize, &bufSize,&dwArg);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				return FAIL;
			}
		    
			/* Check for validity of Data */
		    for( i = 0 ;i< iBuffSize/2 ;i++)
		    {
			    if(sendBuffer[i] != receiveBuffer[i])
			    {
				    status = DSP_EFAIL ;
		        	    ////printf("Data Verification Failed, i=%d: Expected 0x%x Received 0x%x\n", i, sendBuffer[i],receiveBuffer[i]) ;
			    }
		    }

		    if ((k % 500) == 0)
		    {
////			    printf("Thread ID:%d: Issue Reclaim:%d\n",threadID,k) ;
		    }
			    


	    } // End of For loop - iNumTransfers

	/* Free the Allocated Buffers */
	status = DSPStream_FreeBuffers(hStream, arrBuf, DEFNUMBUFS);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_FreeBuffers() for hStream\n");
		}
		////printf("Error in DSPStraem_FreeBuffers()\n");
		return FAIL;
	}

	status = DSPStream_FreeBuffers(hStream1, arrBuf1, DEFNUMBUFS);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_FreeBuffers() for hStream1\n");
		}
		////printf("Error in DSPStraem_FreeBuffers() for hStream1\n");
		return FAIL;
	}

    status = DSPStream_Close(hStream);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_Close() for hStream\n");
		}
		////printf("Error in DSPStraem_Close() for hStream\n");
		return FAIL;
	}

	status = DSPStream_Close(hStream1);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_Close() for hStream1\n");
		}
		////printf("Error in DSPStraem_Close() for hStream1\n");
		return FAIL;
	}

    status = DSPNode_Terminate(hNode, &exitStatus);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Terminate()\n");
		}
		///printf("Error in DSPNode_Terminate\n");
		return FAIL;
	}

    status = DSPNode_Delete(hNode);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Delete()\n");
		}
		////printf("Error in DSPNode_Delete()\n");
		return FAIL;
	}

	OsSpecificMemFree(sendBuffer) ;
    /* Return from this Thread */
    if (DSP_SUCCEEDED(status))
    {
        return PASS;
    }
    else
    {
        return FAIL;
    }
}



/*
 *  ======== main ========
 *
 */
INT multiThreadData_STRESS(INT argc, CHAR * argv[])
{
    	BOOL           			bUseLog=FALSE;
        BOOL           			bVerbose=FALSE;
    	FILE           			*pHandle = NULL; 
	INT            			iNumThreads = DEFNUMTHREADS;
	INT            			iNumTransfers = DEFNUMTXFRS;
    	INT            			iBuffSize = DEFBUFSIZE;
    	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	DWORD		   			threadID[MAXNUMOFTHREADS];
	HANDLE		   			thandle[MAXNUMOFTHREADS];
	MultiThreadArgs 		thrdArgs;
    	struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    	CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
	INT						index=0;
    	INT                 	retVal = PASS;
	INT 		            threadStatus = 0;
	INT 					transferMode = STRMMODE_PROCCOPY ;
	BOOL					fLoadDsp = FALSE;



	 sem_init(&semCreate, 0, 1);

	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Create requested number of Threads and pass the
	   input params to these Threads */
	/* In each of the Thread do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set the Properties, Allocate and Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Do the Data Transfer */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt(argc, argv, "+vl:x:t:n:s:RZP:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("MultiThreadDataTx_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPStream_Issue:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 't':
			/* Number of Threads to be created */
			iNumThreads = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of Threads to be Created = %d\n",iNumThreads);
            TSTLOG_appendLog(pHandle,"Number of Threads to be Created = %d\n",iNumThreads);
			break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
            TSTLOG_appendLog(pHandle, "Number of transfer set to %d",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("Buffer Size set to %d\n",iBuffSize);
            TSTLOG_appendLog(pHandle, "Buffer Size set to %d\n",iBuffSize);
			break;
	case 'R':
            transferMode = STRMMODE_RDMA ;
            ///printf("RDMA Transfer Mode\n") ;
            break ;
        case 'Z':
            transferMode = STRMMODE_ZEROCOPY ;
           /// printf("Zero Copy transfer mode\n") ;
            break ;

	case 'P':
	    transferMode = STRMMODE_ZEROCOPY ;
	   /// printf("Processor Copy transfer mode\n") ;
	    break ;

        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt(argc, argv, "+vl:x:t:n:s:RZP:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    nindex=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(nindex,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = nindex;
            status = DSP_SOK;
            break;
        }
        nindex++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
	    }
    }

	/* Fill the thrdArgs which are to be passed on to the Thread */
	thrdArgs.hProcessor = hProcessor;
	thrdArgs.iNumTransfers = iNumTransfers;
	thrdArgs.iBuffSize = iBuffSize;
	thrdArgs.pHandle = pHandle;
	thrdArgs.bUseLog = bUseLog;
	thrdArgs.bVerbose = bVerbose;
	thrdArgs.transferMode = transferMode ;

	/* Create requested number of Threads and pass the
	   input params to these Threads */

	for (index = 0; index < iNumThreads; index ++)
	{
		sem_wait(&semCreate);
		thrdArgs.ithrdNum = index ;
		thandle[index] = (PVOID) OsSpecificThreadCreate(&threadFxn, &thrdArgs, &threadID[index]);
			
		if (thandle[index] == NULL)
		{
			/* ERROR: Could not create Thread ! Exit !! */
			TST_verbose("ERROR: Could not create Thread ! Exit !! ERROR\n");
		}
	}


	// Wait for the threads to finish before doing a Detach.
	// Code needs to be added here !!! 
	for (index = 0; index < iNumThreads; index ++)
	{
		status = OsSpecificWaitForThread((PVOID)threadID[index],threadStatus);
		
		if(status)
		{
			TSTLOG_appendLog(pHandle, "ERROR: Thread %d Failed\n",index);
			retVal = FAIL;
			TST_verbose("Thread failed %d ",thandle[index]) ;
		}

		TST_verbose("Thread return status %d \n",threadStatus) ;
	}

    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

