/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    wovly0.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\ovly0\
 * 
 *   Desc    A simple program that launches three overlay nodes consecutively.
 *
 *   Assumes:
 *          - DSP program has three nodes with overlay sections.
 *          - DSP nodes have persistent overlay sections (loaded in create,
 *            and not unloaded until delete), that are overlayed at same run address.
 *          - DSP node's create, delete and execute phases are overlayed (prevents
 *            second instance of node being created if node is currently running).
 *
 *   Tests:
 *      - The different nodes can be created, executed, and deleted.
 *      - Second instance of node cannot be created if node is executing,
 *        since create and execute phase are overlayed.
 *      - Another node cannot be created if a node has already been created,
 *        due to persistent overlay sections.
 *
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   OCT 16, 2002    REF=ORG             Vijay Pasam
 *      Original Version adapted from WinCE version.
 *   ============================================================================
 */


#include <getopt_test.h>
#include <dbtype.h>
#include <tst.h>
#include <dbapi.h>

#include <DSPProcessor_OEM.h>

#include <ovly_ti_uuid.h>

/* Hardcoded sample parameters: */
#define DEFAULTBUFSIZE  128     /* Default buffer size in bytes. */
#define MAXBUFSIZE      128
#define DEFAULTDSPUNIT  0       /* Default unit number for the board. */
#define DEFAULTNBUFS    1
#define ARGSIZE         32
#define DFLT_NODES      3
#define MAX_NODES       3
#define DEFNAMESIZE     80



#define DSPEXEC  "ovly0_nortdxidl.dof"

/* dsp task context data structure. */
typedef struct TASK {
    DSP_HPROCESSOR  hProcessor;     /* Handle to processor. */
    struct DSP_UUID        hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE       hNode;          /* Handle to node. */
    DSP_STATUS      createStatus;   /* Status of DSPNode_Create() */
} TASK; 

static DSP_HPROCESSOR   hProc = NULL;

static const struct DSP_UUID *uuid[MAX_NODES] = {&OVLY_TI_uuid0, &OVLY_TI_uuid1,
        &OVLY_TI_uuid2};

static TASK dspTasks[MAX_NODES * 2];

/* Initialization and cleanup routines. */
static DSP_STATUS InitializeNodes(UINT cNodes);
static DSP_STATUS RunTest(UINT cNode);

/* 
 *  ======== overlay_test0_STRESS ========
 */
int overlay_test0_STRESS(int argc, char ** argv)
{
    
    DSP_STATUS      status = DSP_SOK;
    CHAR            *aArgv[DEFNAMESIZE] = {DSPEXEC, "arg1"};
    
    ULONG           ulUnit = DEFAULTDSPUNIT;
    UINT            cNodes = DFLT_NODES;
    int             i = 0;
	int             count;
	INT				tmp_argc;
	CHAR            ** tmp_argv;
	struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;


    tmp_argc = argc;  tmp_argc--;
    argv++;  tmp_argv = argv;

	for (count=0; count < tmp_argc; count++)
	{
		TST_verbose("overlay_test0_STRESS: arg %d : %s\n",count,argv[count]);
	}
	

    /* Open board, show board state, get channel manager, and start tasks: */
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == DSPTYPE_55)
                {
                         TST_verbose("DSP device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&hProc);
        if (DSP_SUCCEEDED(status)) {
            TST_verbose("DSPProcessor_Attach succeeded.\n");
        }
        else {
            TST_verbose("DSPProcessor_Attach failed. Status = 0x%x\n",
                status);
        }
    }
    else
        TST_verbose("Failed to get the desired processor \n");

    if(DSP_SUCCEEDED(status))
     {
        status = DSPProcessor_Load(hProc, 1, (CONST CHAR**)aArgv, NULL);
        if (DSP_FAILED(status)) {
            TST_verbose("DSPProcessor_Load %s failed: 0x%x\n", aArgv[0], status);
        }
        else {
            status = DSPProcessor_Start(hProc);
            if (DSP_FAILED(status)) {
                TST_verbose("DSPProcessor_Start failed: 0x%x\n", status);
            }
        }
    }
    
    if (DSP_SUCCEEDED(status)) {
        /* Allocate Nodes */
        status = InitializeNodes(cNodes);
    }

    for (i = 0; DSP_SUCCEEDED(status) && i < cNodes; i++) {
        status = RunTest(i);
    }

    if (hProc) {
        DSPProcessor_Detach(hProc);
        hProc = NULL;
    }

    if (DSP_SUCCEEDED(status)) {
        TST_verbose("======== TEST PASSED ========\n");
    }
    else {
        TST_verbose("======== TEST FAILED: 0x%x ========\n", status);
    }

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeNodes ========
 */
static DSP_STATUS InitializeNodes(UINT cNodes)
{
    UINT            i;
    struct DSP_CBDATA     *pArgs;
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_NODEATTRIN  attrIn;
    
    DSP_STATUS      status = DSP_SOK;


    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;

    attrIn.uTimeout = DSP_FOREVER;
    attrIn.iPriority = 1;

    for (i = 0; i < MAX_NODES; i++) {
        dspTasks[i].hNode = NULL;
    }

    for (i = 0; i < cNodes; i++) {
        /* Fill in node id */
        pArgs->cData[0] = i;  /* Task id */

        status = DSPNode_Allocate(hProc, uuid[i], pArgs, NULL,
                &(dspTasks[i].hNode));

        if (DSP_FAILED(status)) {
            TST_verbose("DSPNode_Allocate: node %d Failed: 0x%x\n", i, status);
            break;
        }

        status = DSPNode_Allocate(hProc, uuid[i], pArgs, NULL,
                &(dspTasks[i + MAX_NODES].hNode));

        if (DSP_FAILED(status)) {
            TST_verbose("DSPNode_Allocate: node %d Failed: 0x%x\n", i, status);
            break;
        }
        /*
         *  Create all nodes first. This will fail for all but first
         *  nodes, if there is an overlay section that is supposed to
         *  stay in memory throughout all node phases.
         */
        dspTasks[i].createStatus = DSPNode_Create(dspTasks[i].hNode);
        if (DSP_FAILED(dspTasks[i].createStatus)) {
            if (dspTasks[i].createStatus != DSP_EOVERLAYMEMORY) {
                status = dspTasks[i].createStatus;
                break;
            }
        }
    }

    if (DSP_FAILED(status)) {
        for (i = 0; i < cNodes; i++) {
            if (dspTasks[i].hNode) {
                DSPNode_Delete(dspTasks[i].hNode);
            }
        }
    }

    return (status);
}

/*
 *  ======== RunTest ========
 */
static DSP_STATUS RunTest(UINT cNode)
{
    DSP_STATUS  exitStatus = DSP_SOK;
    DSP_STATUS  deleteStatus = DSP_SOK;
    DSP_STATUS  status = DSP_SOK;

    status = dspTasks[cNode].createStatus;
    if (status == DSP_EOVERLAYMEMORY) {
        status = DSPNode_Create(dspTasks[cNode].hNode);
    }

    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(dspTasks[cNode].hNode);
        if (DSP_SUCCEEDED(status)) {
            /*
             *  Make sure another instance of the node cannot be created.
             */
            status = DSPNode_Create(dspTasks[cNode + MAX_NODES].hNode);
            if (status != DSP_EOVERLAYMEMORY) {
                TST_verbose("Failed to prevent overlay memory corruption: "
                    "0x%x\n", status);
            }
            status = DSPNode_Terminate(dspTasks[cNode].hNode, &exitStatus);
            if (DSP_FAILED(status)) {
                TST_verbose("DSPNode_Terminate(nodes[%d]) failed: 0x%x\n",
                        cNode, status);
            }
            if (DSP_FAILED(exitStatus)) {
                TST_verbose("DSPNodeTerminate(nodes[%d]), exitStatus: 0x%x\n",
                    exitStatus);
            }
        }
        else {
            TST_verbose("DSPNode_Run(nodes[%d]) failed: 0x%x\n", cNode, status);
        }
    }
    else {
        TST_verbose("DSPNode_Create(nodes[%d]) failed: 0x%x\n", cNode, status);
    }

    deleteStatus = DSPNode_Delete(dspTasks[cNode].hNode);
    if (DSP_FAILED(deleteStatus)) {
        TST_verbose("DSPNode_Delete(nodes[%d]) failed: 0x%x\n", cNode,
                deleteStatus);
    }

    if (DSP_SUCCEEDED(status)) {
        if (DSP_FAILED(exitStatus)) {
            status = exitStatus;
        }
        else {
            if (DSP_FAILED(deleteStatus)) {
                status = deleteStatus;
            }
        }
    }
    return (status);
}

