/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    wovly1.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\ovly1\
 * 
 *   Desc:   A simple program that launches three overlay nodes consecutively.
 *
 *  Assumes:
 *      - DSP program has three nodes with overlay sections.
 *      - DSP nodes have persistent overlay sections (loaded in create,
 *  and not unloaded until delete), that are overlayed at same run address.
 *      - DSP node's create, delete and execute phases are not overlayed
 *  (allowing asecond instance of node being created if node is currently
 *  running).
 *
 *  Tests:
 *      - The different nodes can be created, executed, and deleted.
 *      - Second instance of node can be created if node is executing,
 *        since create and execute phase are not overlayed.
 *      - Creates and executes two instances of a node, deletes one
 *        of the instances, and then attempts to create another node, to
 *        verify that the create will still fail.
 *
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   OCT 16, 2002    REF=ORG             Vijay Pasam
 *      Original Version adapted from WinCE version.
 *   ============================================================================
 */


#include <stdio.h> 
#include <dbtype.h>
#include <tst.h>
#include <getopt_test.h>

#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <ovly_ti_uuid.h>
#include <testcsl.h>

/* Hardcoded sample parameters: */
#define DEFAULTBUFSIZE  128     /* Default buffer size in bytes. */
#define MAXBUFSIZE      128
#define DEFAULTDSPUNIT  0       /* Default unit number for the board. */
#define DEFAULTNBUFS    1
#define ARGSIZE         32
#define DFLT_NODES      1
#define MAX_NODES       3
#define DEFNAMESIZE     80

#define DSPEXEC  "ovly1_norddxidl.dof"

#define NUMNODES        MAX_NODES * 3


/* dsp task context data structure. */
typedef struct TASK {
    DSP_HPROCESSOR  hProcessor;     /* Handle to processor. */
    struct DSP_UUID        hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE       hNode;          /* Handle to node. */
    DSP_STATUS      createStatus;   /* Status of DSPNode_Create() */
} TASK; 

static DSP_HPROCESSOR   hProc = NULL;

static const struct DSP_UUID *uuid[MAX_NODES] = {&OVLY_TI_uuid0, &OVLY_TI_uuid1,
        &OVLY_TI_uuid2};

static TASK dspTasks[NUMNODES];
static BYTE argsBuf[ARGSIZE + sizeof(ULONG)];

/* Initialization and cleanup routines. */
static DSP_STATUS InitializeNodes(UINT cNodes);
static DSP_STATUS RunTest(UINT cNode);

/* 
 *  ======== main ========
 */
int overlay_test1_STRESS(int argc, char ** argv)
{
    CHAR            chOption;
    DSP_STATUS      status = DSP_SOK;
    CHAR            *aArgv[DEFNAMESIZE] = {DSPEXEC, "arg1"};
    UINT            cNodes = DFLT_NODES;
    int             i = 0;
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;

    

    /* Process options: */
    while((chOption = getopt_test(argc, (char* const)argv, "u:n:")) != (CHAR)EOF) {
        
        switch (chOption) {
            case 'n':
                cNodes = CSL_Atoi(optarg);
                if (cNodes > MAX_NODES) {
                    cNodes = MAX_NODES;
                }
                break;

            default:
                TST_verbose("Unrecognized option: %c\n", chOption);
                break;
        }
    }

    /* Open board, show board state, get channel manager, and start tasks: */
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == DSPTYPE_55)
                {
                        TST_verbose("DSP device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&hProc);
        if (DSP_SUCCEEDED(status)) {
            TST_verbose("DSPProcessor_Attach succeeded.\n");
        }
        else {
            TST_verbose("DSPProcessor_Attach failed. Status = 0x%x\n",
                status);
        }
    }
    else
        TST_verbose("Failed to get the desired processor \n");


    if(DSP_SUCCEEDED(status))
      {
        status = DSPProcessor_Load(hProc, 1, (CONST CHAR**)aArgv, NULL);
        if (DSP_FAILED(status)) {
            TST_verbose("DSPProcessor_Load %s failed: 0x%x\n", aArgv[0], status);
        }
        else {
            status = DSPProcessor_Start(hProc);
            if (DSP_FAILED(status)) {
                TST_verbose("DSPProcessor_Start failed: 0x%x\n", status);
            }
        }
    }

    if (DSP_SUCCEEDED(status)) {
        /* Allocate Nodes */
        status = InitializeNodes(cNodes);
    }

    for (i = 0; DSP_SUCCEEDED(status) && i < cNodes; i++) {
        status = RunTest(i);
    }

    if (hProc) {
        DSPProcessor_Detach(hProc);
        hProc = NULL;
    }

    if (DSP_SUCCEEDED(status)) {
        TST_verbose("======== TEST PASSED ========\n");
    }
    else {
        TST_verbose("======== TEST FAILED: 0x%x ========\n", status);
    }

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeNodes ========
 */
static DSP_STATUS InitializeNodes(UINT cNodes)
{
    UINT            i;
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  attrIn;
    
    DSP_STATUS      status = DSP_SOK;


    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;

    attrIn.uTimeout = DSP_FOREVER;
    attrIn.iPriority = 1;

    for (i = 0; i < NUMNODES; i++) {
        dspTasks[i].hNode = NULL;
    }

    for (i = 0; i < cNodes; i++) {
        /* Fill in node id */
        pArgs->cData[0] = i;  /* Task id */

        status = DSPNode_Allocate(hProc, uuid[i], pArgs, NULL,
                &(dspTasks[i].hNode));

        if (DSP_FAILED(status)) {
            TST_verbose("DSPNode_Allocate: node %d Failed: 0x%x\n", i, status);
            break;
        }

        status = DSPNode_Allocate(hProc, uuid[i], pArgs, NULL,
                &(dspTasks[i + MAX_NODES].hNode));

        if (DSP_FAILED(status)) {
            TST_verbose("DSPNode_Allocate: node %d Failed: 0x%x\n", i, status);
            break;
        }
    }

    if (DSP_FAILED(status)) {
        for (i = 0; i < NUMNODES; i++) {
            if (dspTasks[i].hNode) {
                DSPNode_Delete(dspTasks[i].hNode);
            }
        }
    }

    return (status);
}

/*
 *  ======== RunTest ========
 */
static DSP_STATUS RunTest(UINT cNode)
{
    struct DSP_CBDATA *pArgs;
    DSP_STATUS  exitStatus = DSP_SOK;
    DSP_STATUS  status = DSP_SOK;
    DSP_STATUS  retVal = DSP_SOK;

    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;

    /* Reallocate node if necessary */
    if (dspTasks[cNode].hNode == NULL) {
        status = DSPNode_Allocate(hProc,
            uuid[(cNode) % MAX_NODES], pArgs, NULL,
            &(dspTasks[cNode].hNode));
        if (DSP_FAILED(status)) {
            TST_verbose("DPSNode_Allocate(node[%d]) failed: 0x%x\n", cNode,
                    status);
            retVal = status;
        }
    }

    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(dspTasks[cNode].hNode);
        if (DSP_FAILED(status)) {
            TST_verbose("DSPNode_Create(nodes[%d]) failed: 0x%x\n", cNode,
                    status);
            retVal = status;
        }
    }

    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(dspTasks[cNode].hNode);
        if (DSP_SUCCEEDED(status)) {
            /*
             *  Make sure another instance of the node can be created
             *  while node is running.
             */
            status = DSPNode_Create(dspTasks[cNode + MAX_NODES].hNode);
            if (DSP_FAILED(status)) {
                TST_verbose("Failed to create second node instance: 0x%x\n",
                        status);
                retVal = status;
            }
            else {
                /* Run the second instance */
                status = DSPNode_Run(dspTasks[cNode + MAX_NODES].hNode);
                if (DSP_FAILED(status)) {
                    TST_verbose("Failed to run second node instance: 0x%x\n",
                        status);
                    retVal = status;
                }
                else {
                    status = DSPNode_Terminate(
                            dspTasks[cNode + MAX_NODES].hNode, &exitStatus);
                    if (DSP_FAILED(status)) {
                        TST_verbose("DSPNode_Terminate(nodes[%d]) failed: "
                            "0x%x\n", cNode + MAX_NODES, status);
                        retVal = status;
                    }
                    if (DSP_FAILED(exitStatus)) {
                        TST_verbose("DSPNodeTerminate(nodes[%d]), exitStatus: "
                            "0x%x\n", cNode + MAX_NODES, exitStatus);
                        retVal = status;
                    }
                }
                status = DSPNode_Delete(dspTasks[cNode + MAX_NODES].hNode);
                if (DSP_FAILED(status)) {
                    TST_verbose("DSPNode_Delete(nodes[%d]) failed: 0x%x\n",
                            cNode + MAX_NODES, status);
                    retVal = status;
                }
                else {
                    dspTasks[cNode + MAX_NODES].hNode = NULL;
                }
                /* Make sure reference counts are still positive */
                if (dspTasks[cNode + 1].hNode == NULL) {
                    status = DSPNode_Allocate(hProc,
                            uuid[(cNode + 1) % MAX_NODES], pArgs, NULL,
                            &(dspTasks[cNode + 1].hNode));
                    if (DSP_FAILED(status)) {
                        TST_verbose("DPSNode_Allocate(node[%d]) failed: 0x%x\n",
                            cNode + 1, status);
                        retVal = status;
                    }
                }
                if (DSP_SUCCEEDED(status)) {
                    /* Attempt to create node */
                    status = DSPNode_Create(dspTasks[cNode + 1].hNode);
                    if (status != DSP_EOVERLAYMEMORY) {
                        TST_verbose("DSPNode_Create(nodes[%d]) failed to "
                            "return correct error code. Status: 0x%x\n",
                            cNode + 1, status);
                        retVal = status;
                    }
                    status = DSPNode_Delete(dspTasks[cNode + 1].hNode);
                    if (DSP_FAILED(status)) {
                        TST_verbose("DSPNode_Delete(nodes[%d]) failed: 0x%x\n",
                            cNode + 1, status);
                        retVal = status;
                    }
                    dspTasks[cNode + 1].hNode = NULL;
                }
            }

            status = DSPNode_Terminate(dspTasks[cNode].hNode, &exitStatus);
            if (DSP_FAILED(status)) {
                TST_verbose("DSPNode_Terminate(nodes[%d]) failed: 0x%x\n",
                        cNode, status);
                retVal = status;
            }
            if (DSP_FAILED(exitStatus)) {
                TST_verbose("DSPNodeTerminate(nodes[%d]), exitStatus: 0x%x\n",
                    exitStatus);
                retVal = status;
            }
        }
        else {
            TST_verbose("DSPNode_Run(nodes[%d]) failed: 0x%x\n", cNode, status);
            retVal = status;
        }
    }

    status = DSPNode_Delete(dspTasks[cNode].hNode);
    dspTasks[cNode].hNode = NULL;
    if (DSP_FAILED(status)) {
        TST_verbose("DSPNode_Delete(nodes[%d]) failed: 0x%x\n", cNode, status);
        retVal = status;
    }

    return (retVal);
}

