/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPProcessor_Attach.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPProcessor\DSPProcessor_Attach.c
 * 
 *   Desc:
 *  Description:
 *      DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *        DSPProcessor_Attach(UINT uProcessor, DSP_PROCESSORATTRIN *pAttrIn,
 *                            DSP_HPROCESSOR *phProcessor);
 *  Usage:
 *        DSPProcessor_Attach [-v] [-l "log path/filename"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *  Issues:
 *        First call to DSPProcessor_Attach returns DSP_SALREADYATTACHED,
 *        but don't know why!  All succeeding calls return correct DSP_SOK.
 *        To fix, we call Attach prior to actual test run.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>

#define DEFNAMESIZE    80

/*
 *  ======== main ========
 *
 */
INT DSPProcessor_Attach_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT         opt;
    INT         i, j, k;
    INT         cRetVal = 1;
    BOOL        bUseLog=FALSE;
    FILE        * pHandle = NULL;
    CHAR        szLogName[DEFNAMESIZE];
    DSP_STATUS  status;

    /* INPUT PARAMETERS */
    UINT arg1[] =   /* arg1: number of processor to attach to */
    {
        0,              /* Valid number */
        50,             /* Invalid number out of range*/
    };

    struct DSP_PROCESSORATTRIN *arg2[] =   /* arg2: attributes to apply */
    {
        NULL,           /* valid pointer to attributes to be allocated */
        NULL            /* valid NULL pointer for default attributes */
    };

    /* Can't test invalid pointer here. */
    DSP_HPROCESSOR arg3[] = /* arg3: processor handle */
    {
        NULL,           /* valid handle to be set */
        NULL            /* invalid NULL pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(UINT))
    #define NUMARG2 (sizeof(arg2)/sizeof(struct DSP_PROCESSORATTRIN *))
    #define NUMARG3 (sizeof(arg3)/sizeof(DSP_HPROCESSOR))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] =
    {
        {
            {
                {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0] */
                {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][1] */
            },
            {
                {DSP_SOK, TST_SHOULDEQUAL}, /* [0][1][0] */
                {DSP_SOK, TST_SHOULDEQUAL}, /* [0][1][1] */
            },

        },
        {
            {
                {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][0][0] */
                {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][0][1] */
            },
            {
                {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][1][0] */
                {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][1][1] */
            },
        },
    };

    /* Allocate argument pointer */
    arg2[0] = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    arg2[0]->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    arg2[0]->uTimeout = 50;

    /* Reset getopt pointer */
    optind = 0;

    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPProcessor_Attach: Verbose Mode On\n");
            break;

        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;

        case 'x':
            /* Use DSP executable */
            
            /*
             *  Not used in this test - SKIP
             *  Leave option here, however, in case executable argument
             *  is passed in from 'apitst'
             */
            break;

        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                    "Invalid Option Specified.\n");
            }

            break;
        }
    }

    TST_verbose("DSPProcessor_Attach: ");
    TST_verbose("%i: ", iRunTime);

    if (cRetVal > 0) {
        
        for (i = 0; i < NUMARG1; i++) {
            
            for (j = 0; j < NUMARG2; j++) {
                
                for (k = 0; k < NUMARG3; k++) {
                    
                    status = DSPProcessor_Attach(arg1[i], arg2[j], &arg3[k]);
                    
                    if (!TST_CheckVal(status,
                        checkArray[i][j][k].dwValue,
                        checkArray[i][j][k].fEqualFlag)) {
                        cRetVal = 0;
                        TST_verbose("Attach [%d] [%d] [%d] returned %x\n",
                            i, j, k, status);
                        
                        if (bUseLog) {
                            TSTLOG_appendLog(pHandle, "FAIL\n");
                            TSTLOG_appendLog(pHandle,
                                "DSPProcessor_Attach [%d][%d][%d] failed.\n",
                                i, j, k);
                        }
                    }
                    
                    if (DSP_SUCCEEDED(status)) {
                        DSPProcessor_Detach(arg3[k]);
                    }
                }
            }
        }
    }


	OsSpecificMemFree(arg2[0]);

    if (cRetVal != 0) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}

