/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPProcessor_Start.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPProcessor\DSPProcessor_Start.c
 * 
 *   Desc:
 *        DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *        DSPProcessor_Start(DSP_HPROCESSOR hProcessor);
 *  Usage:
 *        DSPProcessor_Start [-v] [-l "log path/filename"] [-x "valid executable"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: name of valid DSP executable
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC         "dataxfr_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPProcessor_Start_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            iArgc = 1;
    INT            i;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    DSP_STATUS        status;
    struct DSP_PROCESSORATTRIN *pAttrIn;


    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =       /* arg1: handle of attached processor */
    {
        NULL, /* Also a valid handle (but to test wrong state) */
        NULL, /* Valid handle to be attached */
        NULL, /* Invalid handle */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] =
    {
        /* Should be DSP_EWRONGSTATE? */
        {DSP_EWRONGSTATE, TST_SHOULDEQUAL}, /* [0] */
        {DSP_SOK, TST_SHOULDEQUAL}, /* [1] */
        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [2] */
    };

    /* Allocate pointer to dummy set of attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPProcessor_Start: Verbose Mode On\n");
            break;
    case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));

         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);

            TSTLOG_appendLog(pHandle, "DSPProcessor_Start:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
        case 'x':
            /* use given executable */
        aArgv[0] = (char *) optarg;
            break;
        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
    }

    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPProcessor_Start: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &arg1[0]);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
       if (!DSP_SUCCEEDED(status)) {
           /* Report Error */
           cRetVal = 0;

           if (bUseLog) {
               TSTLOG_appendLog(pHandle, "FAIL\n");
               TSTLOG_appendLog(pHandle,
               "Error in DSPProcessor_Attach()\n");
           }
           break;
       }

       arg1[1] = arg1[0];


       for (i = 0; i < NUMARG1; i++) {

            /* Don't load processor first iteration to test wrong
               state return value.  Only load during 2nd iteration
               because it only needs to be loaded once */
            if (i == 1) {
                /* Load Processor with Executable */
                status = DSPProcessor_Load(arg1[0], iArgc, (CONST CHAR **)aArgv, 
								NULL);
                if (!DSP_SUCCEEDED(status)) {
                    /* Report Error */
                    cRetVal = 0;

                    if (bUseLog) {
                        TSTLOG_appendLog(pHandle, "FAIL\n");
                        TSTLOG_appendLog(pHandle,
                        "Error in DSPProcessor_Load()\n");
                    }
                    break;
                }
            }

            status = DSPProcessor_Start(arg1[i]);

            if (!TST_CheckVal(status,
                checkArray[i].dwValue,
                checkArray[i].fEqualFlag)) {
                cRetVal = 0;
                TST_verbose("Start [%d] returned %x\n",
                             i, status);
                if (bUseLog) {
                    TSTLOG_appendLog(pHandle, "FAIL\n");
                    TSTLOG_appendLog(pHandle,
                    "DSPProcessor_Start [%d] failed.\n", i);
                }
            }
       }

       /* Detach Processor */
       status = DSPProcessor_Detach(arg1[0]);

       break;
    }


      OsSpecificMemFree(pAttrIn);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    return 0;
}

